#!/usr/bin/perl
#
# Version: 1.5
# 17.02.2018
# Copyright (C) 2018 by LuftEngineering GmbH, Germany
# Report bugs to:  development@luft-it
#
use warnings;
use strict;

$ENV{"PATH"}='';
$ENV{"BASH_ENV"}='';
$ENV{"ENV"}='';

# Please change this path
use lib "/usr/share/perl5/Nagios";

use POSIX;
use Nagios::Plugin;
use Nagios::Plugin::Functions qw(:codes %ERRORS %STATUS_TEXT @STATUS_CODES);
use Data::Dumper;

my $np = Nagios::Plugin->new(
        plugin  => "check_arrconf",
	shortname => "RAID",
	usage => "check_arcconf.pl --path <Path to arcconf>",
	blurb => "Monitors integrity of a RAID System with arcconf",
        version => 'Version: 1.5',
        url     => "www.luft-it.de.de/freeware",
);

$np->add_arg(
    spec => 'path=s', 
    help => "Path to arcconf (Default: '/usr/StorMan/arcconf')",
    default => "/usr/StorMan/arcconf",
);

$np->add_arg(
    spec => 'cmdArgs=s', 
    help => "Args for arcconf (Default: 'getconfig 1 al')",
    default => "getconfig 1 al",
);

$np->add_arg(
    spec => 'pipe', 
    help => "Read RAID status from STDIN"
);

# Kommandozeilen Argumente auswerten
$np->getopts();

my $path = $np->opts->{"path"};
my $args = $np->opts->{"cmdArgs"};
my $debug = $np->opts->{"verbose"};
my $pipe = $np->opts->{"pipe"};

if ($debug) {
 printf ("\n[Command line options]\n");
 foreach my $i (sort(keys %{$np->opts})) {
  my $arg = $i;
  my $value = "undef";
  $value = $np->opts->{$i} if defined ($np->opts->{$i});
  printf (" >> %-15s = %s\n",$arg,$value) if $arg !~ m/(_)/;
 }
 print "\n"
}

# Timeout starten
alarm $np->opts->{"timeout"};

my $arcconf = "";

if ($pipe) {
 while (<STDIN>) {
  $arcconf .= $_;
 }
} else {
  $np->nagios_exit(UNKNOWN,"$path is a directory") if (-d $path); 
  if (-x $path) {
 # arcconf ausführe
  my $cmd = "$path $args";
  $arcconf = `$cmd`;
  } else {
   $np->nagios_exit(UNKNOWN,"$path is not executable"); 
  }
}

if ($debug >=2 ) {
 print "[Output arcconf]\n";
 print $arcconf;
 print "----------------------------------------------------------------------\n\n";
}

$np->nagios_exit(UNKNOWN,"No output from arcconf") if length($arcconf) == 0;

my @controllers;
my @logDrives;
my @phyDisks;

# Ausgabe von arcconf auswerten
my @lines = split("\n",$arcconf);

foreach my $line (@lines) {
# Leerzeichen am Zeilenanfang entfernen
 $line =~ s/^\s+//g;
# Leerzeichen vor : entfernen
 $line =~ s/\s+: /:/g;

 $line =~ s/[^A-Za-z0-9#\:\.\-_\s]//g;

# Controller
 if ($line =~ m/^Controller Status/) {
  my %controller;
  $controller{"status"} = (split(":",$line))[1];
  push(@controllers,\%controller);
 } elsif ($line =~ m/^Controller Model:/) {
   my %controller = %{pop(@controllers)};
   $controller{"model"} = (split(":",$line))[1];
   push(@controllers,\%controller);
 } elsif ($line =~ m/^Physical Slot:/) {
   my %controller = %{pop(@controllers)};
   $controller{"slot"} = (split(":",$line))[1];
   push(@controllers,\%controller);
 } elsif ($line =~ m/^Status:[^of logical device:]/) {
   my %controller = %{pop(@controllers)};
   $controller{"battery"} = (split(":",$line))[1];
   push(@controllers,\%controller);
 } elsif ($line =~ m/^Time remaining/) {
   my %controller = %{pop(@controllers)};
   $controller{"runtime"} = (split(":",$line))[1];
   push(@controllers,\%controller);
 }


# Logische Laufwerke
 elsif ($line =~ m/^Logical [Dd]evice name:/) {
   my %logDrive;
   $logDrive{"name"} = (split(":",$line))[1];
   push(@logDrives,\%logDrive);
 } elsif ($line =~ m/^RAID level:/) {
   my %logDrive = %{pop(@logDrives)};
   $logDrive{"RAID"} = (split(":",$line))[1];
   push(@logDrives,\%logDrive);
 } elsif ($line =~ m/^Status of [Ll]ogical [Dd]evice:/) {
   my %logDrive = %{pop(@logDrives)};
   $logDrive{"status"} = (split(":",$line))[1];
   push(@logDrives,\%logDrive);
 }

# Physikalische Laufwerke
 elsif ($line =~ m/^Device #/) {
   my %phyDisk;
# Init Hash
   $phyDisk{"serialNumber"} = "MISSING";
   $phyDisk{"status"} = "MISSING";
   $phyDisk{"channel"} = "MISSING";

   $phyDisk{"name"} = (split("#",$line))[1];
   $phyDisk{"isHardDisk"} = 0;
   push(@phyDisks,\%phyDisk);
 } elsif ($line =~ m/^Device is a Hard drive/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"isHardDisk"} = 1;
   push(@phyDisks,\%phyDisk); 
 } elsif ($line =~ m/^State:/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"status"} = (split(":",$line))[1];
   push(@phyDisks,\%phyDisk); 
 } elsif ($line =~ m/^Serial number:/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"serialNumber"} = (split(":",$line))[1];
   push(@phyDisks,\%phyDisk); 
 } elsif ($line =~ m/^Reported Channel,Device\(T:L\):/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"channel"} = (split(":",$line,3))[2];
   push(@phyDisks,\%phyDisk);
 } elsif (($line =~ m/^Reported Channel,Device:/) or ($line =~ m/^Reported Enclosure,Slot/)) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"channel"} = (split(":",$line))[1];
   push(@phyDisks,\%phyDisk);
 } elsif ($line =~ m/^Vendor:/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"vendor"} = (split(":",$line))[1];
   $phyDisk{"vendor"} = "MISSING" if !defined($phyDisk{"vendor"});
   push(@phyDisks,\%phyDisk);
 } elsif ($line =~ m/^Model:/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"model"} = (split(":",$line))[1];
   $phyDisk{"model"} = "MISSING" if !defined($phyDisk{"model"});
   push(@phyDisks,\%phyDisk);
 } elsif ($line =~ m/^S\.M\.A\.R\.T\.:/) {
   my %phyDisk = %{pop(@phyDisks)};
   $phyDisk{"SMART"} = (split(":",$line))[1];
   push(@phyDisks,\%phyDisk);
 }  
}

# Ausgabe für Nagios vorbereiten

print Dumper(@phyDisks) if ($debug);

# Auswertung Controller
foreach my $ctrl (@controllers) {
 if ($debug) {
  print "[Controller]\n";
  foreach my $prop (sort (keys(%{$ctrl}))) {
   printf (" >> %-15s = %s\n",$prop,$ctrl->{$prop});
  }
  print "\n";
 }
 my $msg = "Ctrl: ".$ctrl->{"model"}." - Slot: ".$ctrl->{"slot"}." (".$ctrl->{"status"}.")";
 
 if (exists($ctrl->{"battery"})) {
  if ($ctrl->{"battery"} ne "Not Installed") {
   my $runtime = $ctrl->{"runtime"};
   $runtime =~ s/ hours/h/;
   $runtime =~ s/ minutes/min/;
   $msg .= " - Battery (".$ctrl->{"battery"}.") Runtime: $runtime";
  }
 }

 if (($ctrl->{"status"} eq "Optimal") or ($ctrl->{"status"} eq "OK")) {
  $np->add_message(OK,$msg);
 } else {
  $np->add_message(CRITICAL,$msg);
 }
}

# Auswertung logische Laufwerke
foreach my $log (@logDrives) {
 if ($debug) {
  print "[Logical Devices]\n";
  foreach my $prop (sort (keys(%{$log}))) {
   printf (" >> %-15s = %s\n",$prop,$log->{$prop});
  }
  print "\n";
 }
 my $msg = "Drive: ".$log->{"name"}." [RAID ".$log->{"RAID"}."] (".$log->{"status"}.")";
 if (($log->{"status"} eq "Optimal") or ($log->{"status"} eq "OK")) {
  $np->add_message(OK,$msg);
 } elsif ($log->{"status"} eq "Degraded") { 
  $np->add_message(WARNING,$msg);
 } else {
  $np->add_message(CRITICAL,$msg);
 }
}

# Auswertung physikalische Laufwerke
foreach my $phy (@phyDisks) {
 if ($debug) {
  print "[Physical Disk]\n";
  foreach my $prop (sort (keys(%{$phy}))) {
   printf (" >> %-15s = %s\n",$prop,$phy->{$prop});
  }
  print "\n";
 }
 if ($phy->{"isHardDisk"}) {
  my $msg = "Disk: ".$phy->{"name"}." (".$phy->{"status"}.") S.M.A.R.T.: ".$phy->{"SMART"} ;
  if (($phy->{"status"} eq "Online") or ($phy->{"status"} eq "Ready") or ($phy->{"status"} eq "Hot Spare") or ($phy->{"status"} eq "Global Hot-Spare") or ($phy->{"status"} eq "Dedicated Hot-Spare") or ($phy->{"status"} eq "JBOD") or or ($phy->{"status"} eq "Raw Pass Through") and ($phy->{"SMART"} eq "No")) {
   $np->add_message(OK,$msg);
  } else {
   $msg .= " - Channel: ".$phy->{"channel"}." - Model: ".$phy->{"vendor"}." ".$phy->{"model"}." - SNR: ".$phy->{"serialNumber"};
   $np->add_message(CRITICAL,$msg);
  }
 }
}

# Timeout stoppen
alarm 0;

# Exit Script
my ($code, $output) = $np->check_messages(join => " - ",join_all => " - ");
$np->nagios_exit($code,$output);
